import math
import types

def fd( f ):
	i = math.modf( f * 1000 )
	return i[1]/1000

### Vector ###
class Vector:
	def __init__(v,*arg):
		if len(arg) == 1:# and type(arg[0]) == 'Vector':
			v.x = arg[0][0]
			v.y = arg[0][1]
			v.z = arg[0][2]
		elif len(arg) == 3:
			v.x = arg[0]
			v.y = arg[1]
			v.z = arg[2]
		else:
			v.x = v.y = v.z = 0.
	
	def Normalize(v):
		s = math.sqrt( v.x * v.x + v.y * v.y + v.z * v.z )
		if s != 0.0: s = 1.0 / s
		r = Vector()
		r.x = v.x * s
		r.y = v.y * s
		r.z = v.z * s
		return r
	
	def Dot(v,a):
		return v.x * a.x + v.y * a.y + v.z * a.z
	
	def Cross(v,a):
		return Vector( v.y * a.z - v.z * a.y,  v.z * a.x - v.x * a.z, v.x * a.y - v.y * a.x )
	
	def Length(v):
		return math.sqrt( v.x * v.x + v.y * v.y + v.z * v.z )
	
	def Max(v):
		r = v.x
		if r < v.y:
			r = v.y
		if r < v.z:
			r = v.z
		return r
	
	def FromBlender(self,b):
		self.x = fd(b.x)
		self.y = fd(b.y)
		self.z = fd(b.z)
		return
	
	def ToBlender(self,b):
		b[0] = self.x
		b[1] = self.y
		b[2] = self.z
	
	def Lerp(self,r,w):
		ret = Vector()
		ret.x = self.x + ( r.x - self.x )*w
		ret.y = self.y + ( r.y - self.y )*w
		ret.z = self.z + ( r.z - self.z )*w
		return ret
	
	def __repr__(self):
		return "Vector["+str(self.x)+", "+str(self.y)+", "+str(self.z)+"]"
	
	def __add__(v,*r):
		return v.__operation( (lambda a,b: a+b), r )
	
	def __sub__(v,*r):
		return v.__operation( (lambda a,b: a-b), r )
	
	def __mul__(v,*r):
		return v.__operation( (lambda a,b: a*b), r )
	
	def __div__(v,*r):
		return v.__operation( (lambda a,b: a/b), r )
	
	def __operation(v,f,r):
		if len(r) == 3:
			inc = 1; arg = r
		elif hasattr(r[0],"__len__"):
			inc = 1; arg = r[0]
		else:
			inc = 0; arg = r
		ret= Vector()
		for i in range(3):
			ret[i] = f( v[i], arg[inc*i] )
		return ret
	
	def __neg__(v):
		return Vector( -v.x, -v.y, -v.z )
	
	def __cmp__(v,a):
		if v[0]==a[0] and v[1]==a[1] and v[2]==a[2]:
			return 0
		tl = v.Length()
		al = a.Length()
		if tl > al:
			return 1
		else:
			return -1
	
	def __len__(v):
		return 3
	
	def __getitem__(v,i):
		if i==0:
			return v.x
		if i==1:
			return v.y
		if i==2:
			return v.z
		return None
	
	def __setitem__(v,i,n):
		n= float(n)
		if i==0:
			v.x = n
		if i==1:
			v.y = n
		if i==2:
			v.z = n
		else:
			return 0
		return 1

def QuadricIPO( v1, v2, v3, t ):
	it = 1.0 - t
	ret = v1*(it*it) + v2*(2*t*it) + v3 * (t*t);
	return ret;
	
### Matrix ###
class Matrix:
	def __init__(m):
		m.Identity()
	
	def Identity(m):
		m.mat = [0.0] * 16
		m.mat[0] = m.mat[5] = m.mat[10] = m.mat[15] = 1.0
	
	def __mul__(m,r):
		ret = Matrix()
		ct = 0
		for i in range(4):
			ct = i << 2
			
			ret.mat[ct + 0 ] =	m.mat[ ct + 0 ] * r.mat[ 0 ] + m.mat[ ct + 1 ] * r.mat[ 4 ] + m.mat[ ct + 2 ] * r.mat[ 8 ] + m.mat[ ct + 3 ] * r.mat[ 12]
			ret.mat[ct + 1 ] =	m.mat[ ct + 0 ] * r.mat[ 1 ] + m.mat[ ct + 1 ] * r.mat[ 5 ] + m.mat[ ct + 2 ] * r.mat[ 9 ] + m.mat[ ct + 3 ] * r.mat[ 13]
			ret.mat[ct + 2 ] =	m.mat[ ct + 0 ] * r.mat[ 2 ] + m.mat[ ct + 1 ] * r.mat[ 6 ] + m.mat[ ct + 2 ] * r.mat[ 10] + m.mat[ ct + 3 ] * r.mat[ 14]
			ret.mat[ct + 3 ] =	m.mat[ ct + 0 ] * r.mat[ 3 ] + m.mat[ ct + 1 ] * r.mat[ 7 ] + m.mat[ ct + 2 ] * r.mat[ 11] + m.mat[ ct + 3 ] * r.mat[ 15]
		
		return ret
	
	def __invert__(m):
		ret = Matrix()
		ret.mat[0] = m.mat[0]
		ret.mat[1] = m.mat[4]
		ret.mat[2] = m.mat[8]
		
		ret.mat[4] = m.mat[1]
		ret.mat[5] = m.mat[5]
		ret.mat[6] = m.mat[9]
		
		ret.mat[8] = m.mat[2]
		ret.mat[9] = m.mat[6]
		ret.mat[10]= m.mat[10]
		
		ret.mat[12]= -(m.mat[12]*m.mat[0]+m.mat[13]*m.mat[4]+m.mat[14]*m.mat[8])
		ret.mat[13]= -(m.mat[12]*m.mat[1]+m.mat[13]*m.mat[5]+m.mat[14]*m.mat[9])
		ret.mat[14]= -(m.mat[12]*m.mat[2]+m.mat[13]*m.mat[6]+m.mat[14]*m.mat[10])
		
		return ret
	
	def Transform(m,v):
		ret = Vector()
		ret.x = v.x * m.mat[0] + v.y * m.mat[4] + v.z * m.mat[8] + m.mat[12]
		ret.y = v.x * m.mat[1] + v.y * m.mat[5] + v.z * m.mat[9] + m.mat[13]
		ret.z = v.x * m.mat[2] + v.y * m.mat[6] + v.z * m.mat[10]+ m.mat[14]
		return ret

	
	def Normalize(m):
		v = Vector()
		for i in range(3):
			for j in range(3):
				v[j] = m[i*4+j]
			v = v.Normalize()
			for j in range(3):
				m[i*4+j] = v[j]
		return m
	
	def FromAxisAngle(m,v,a):
		cos = math.cos(a)
		sin = math.sin(a)
		mcs = 1.0 - cos
		xx = v.x * v.x
		yy = v.y * v.y
		zz = v.z * v.z
		mxy= v.x * v.y * mcs
		mxz= v.x * v.z * mcs
		myz= v.y * v.z * mcs
		sx = v.x * sin
		sy = v.y * sin
		sz = v.z * sin
		
		m.mat[0] = xx * mcs + cos
		m.mat[1] = mxy - sz
		m.mat[2] = mxz + sy
		m.mat[4] = mxy + sz
		m.mat[5] = yy * mcs + cos
		m.mat[6] = myz - sx
		m.mat[8] = mxz - sy
		m.mat[9] = myz + sx
		m.mat[10]= zz * mcs + cos
	
	def FromQuaternion(m,q):
		xx = q.x * q.x
		yy = q.y * q.y
		zz = q.z * q.z
		xy = q.x * q.y
		xz = q.x * q.z
		yz = q.y * q.z
		wx = q.w * q.x
		wy = q.w * q.y
		wz = q.w * q.z
		
		m.mat[0] = 1 - 2 * ( yy + zz );
		m.mat[1] = 2 * ( xy + wz );
		m.mat[2] = 2 * ( xz - wy );
		m.mat[4] = 2 * ( xy - wz );
		m.mat[5] = 1 - 2 * ( xx + zz );
		m.mat[6] = 2 * ( yz + wx );
		m.mat[8] = 2 * ( xz + wy );
		m.mat[9] = 2 * ( yz - wx );
		m.mat[10] = 1 - 2 * ( xx + yy );
	
	def FromBlender(m,b):
		for i in range(4):
			ii = i *4
			for j in range(4):
				m.mat[ ii + j ] = fd(b[i][j])
	
	def __repr__(m):
		return "Matrix"+str(m.mat)
	
	def __getitem__(m,i):
		return m.mat[i]
	
	def __setitem__(m,i,n):
		m.mat[i] = n
	
	def __len__(m):
		return 16

class Quaternion:
	def __init__(q,*arg):
		if len(arg) == 2:
			q.RotationArc(arg[0],arg[1])
		else:
			q.x = q.y = q.z = q.w = 0.
	
	def RotationArc(q,v1,v2):
		v = v1.Cross( v2 )
		s = math.sqrt( (1.0 + v1.Dot(v2)) * 2.0 )
		a = 1.0 / s
		
		q.x = v.x * a
		q.y = v.y * a
		q.z = v.z * a
		q.w = s / 2.0
	
	def __repr__(q):
		return "Quaternion x:%f,y:%f,z:%f,w:%f" % (q.x,q.y,q.z,q.w)
	
	def slerp(q,o,t):
		nor = q.x*o.x + q.y*o.y + q.z*o.z + q.w*o.w
		flip = 0
		if nor < 0:
			nor = -nor
			flip = 1
		if 1.0 - nor < 0.001:
			inv = 1.0 - t
		else:
			th = math.acos(nor)
			s = 1.0 / math.sin(th)
			inv = math.sin(( 1.0 - t )*th)* s
			t = math.sin( t* th )+s
		
		if flip:
			t = -t
		ret = Quaternion()
		ret.x = inv * q.x + t * o.x
		ret.y = inv * q.y + t * o.y
		ret.z = inv * q.z + t * o.z
		ret.w = inv * q.w + t * o.w
		return ret