"""
	Subdivider Module
"""

import math
import Blender
import math3d
import nmeshx

from nmeshx import *



def __doTR( me, vs, lv ):
	ms = me.getEdge(vs[0],vs[1]).calcuMidPoint()
	ret= [[vs[0],ms,vs[2]],[ms,vs[1],vs[2]]]
	if not lv:return ret
	return __doTR( me, ret[0], lv-1) + __doTR( me, ret[1], lv-1)

def __doTS( me, vs, lv ):
	ms = [me.getEdge(vs[0],vs[1]).calcuMidPoint(), me.getEdge(vs[1],vs[2]).calcuMidPoint()]
	ret= [[vs[0],ms[0],ms[1],vs[2]],[ms[0],vs[1],ms[1]]]
	if not lv:return ret
	return __doQS( me, ret[0],lv-1) + __doTS( me, ret[1],lv-1)

def __doQS( me, vs, lv ):
	ms = [me.getEdge(vs[0],vs[1]).calcuMidPoint(), me.getEdge(vs[2],vs[3]).calcuMidPoint()]
	ret= [[vs[0],ms[0],ms[1],vs[3]],[ms[0],vs[1],vs[2],ms[1]]]
	if not lv:return ret
	return __doQS( me, ret[0], lv-1) + __doQS( me, ret[1], lv-1)

def __doQP( me, vs, lv ):
	ms = [me.getEdge(vs[0],vs[1]).calcuMidPoint(), me.getEdge(vs[1],vs[2]).calcuMidPoint()]
	mp = me.addVertex( getMedianPoint( vs ) )
	ret= [[vs[0],ms[0],mp,vs[3]],[vs[3],mp,ms[1],vs[2]],[ms[0],vs[1],ms[1],mp]]
	if not lv:return ret
	return __doQS( me, ret[0],lv-1) + __doQS( me, ret[1],lv-1) + __doQP( me, ret[2],lv-1)

def __doALL( me, vs, lv ):
	nv = len(vs)
	ct = me.addVertex( getMedianPoint( vs ) )
	ms = [me.getEdge(vs[i], vs[(i+1)%nv]).calcuMidPoint() for i in range(nv)]
	ret = [[vs[i], ms[i], ct, ms[(i+nv-1)%nv]] for i in range(nv)]
	if not lv:return ret
	r=[]
	for ar in ret:
		r+= __doALL( me, ar, lv-1 )
	return r

def doSubdivide( mesh, level, smooth = 0 ):
	tmp_faces = mesh.faces[:]
	del_faces = []
	for face in tmp_faces:
		if not face.isSelected():continue
		
		nfe = len(face.edges)
		llk = []
		for id in range(nfe):
			of = face.getAdjacentFace(id)
			if of and of.isSelected():
				llk.append( id )
		
		if not len(llk):continue
		
		def defSort(v,l):
			return v[l:] + v[0:l]
		
		newfc = []
		if nfe == 3 and len(llk) == 1:
			newfc = __doTR( mesh, defSort( face.bface.v, llk[0]), level-1 )
		elif len(llk) < 3:
			if nfe == 3:
				if llk[0] == 0 and llk[1] == 2:
					llk = [2,0]
				newfc = __doTS( mesh, defSort( face.bface.v, llk[0]), level-1 )
			elif nfe == 4:
				if len(llk) == 1:
					newfc = __doQS( mesh, defSort( face.bface.v, llk[0]), level-1 )
				elif ([x==1 or x==3 for x in [math.fabs(llk[0]-llk[1])]][0]):
					deft = llk[ llk[1] - llk[0] != 1 ]
					newfc = __doQP( mesh, defSort( face.bface.v, deft), level-1 )
				else:
					newfc = __doQS( mesh, defSort( face.bface.v, llk[0]), level-1 )
		else:
			newfc = __doALL( mesh, face.bface.v, level-1 )
		
		if not newfc:continue
		for e in newfc:
			nf = mesh.addFaceFromVertices( e )
			nf.bface.mat = face.bface.mat
			nf.bface.uv = face.bface.uv
			nf.bface.col = face.bface.col
		del_faces.append( face )
	[mesh.deleteFace( f ) for f in del_faces]
	mesh.toBlender()

class Subdivider:
	def __init__( self, mesh, smooth = 0 ):
		self.me = mesh
		self.issmooth = smooth
	
	def __TR( s, f ):
		return
	def __TS( s, f ):
		return
	def __QS( s, f ):
		return
	def __QP( s, f ):
		return
	def __ALL( s, f ):
		return